import 'package:flutter/material.dart';
import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:flutter_chat_ui_kit_example/utils/constants.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:integration_test/integration_test.dart';

// import 'package:flutter_chat_ui_kit_example/main.dart' as app;

void main() {
  IntegrationTestWidgetsFlutterBinding.ensureInitialized();
  Duration buffer = const Duration(seconds: 5);

  group('end-to-end test', () {
    test('initializing CometChat and logging in', () async {
      //populating auth settings
      UIKitSettings uiKitSettings = (UIKitSettingsBuilder()
            ..subscriptionType = CometChatSubscriptionType.allUsers
            ..region = CometChatConstants.region
            ..autoEstablishSocketConnection = true
            ..appId = CometChatConstants.appId
            ..authKey = CometChatConstants.authKey)
          .build();
      print('auth settings populated');

      //initializing CometChat
      await CometChatUIKit.init(
          uiKitSettings: uiKitSettings,
          onSuccess: (String successMessage) {
            debugPrint(
                "Initialization completed successfully  $successMessage");
          },
          onError: (CometChatException e) {
            debugPrint("Initialization failed with exception: ${e.message}");
          });

      //user login
      await CometChatUIKit.login('superhero1');
      // await tester.pumpAndSettle(buffer);
      print('went to dashboard?');

      //check if the user has been logged in
      expect((await CometChatUIKit.getLoggedInUser())?.uid, 'superhero1');
      print('found supehero1');
    });

    testWidgets('launch Users with messages', (tester) async {
      //launch CometChatUsersWithMessages
      await tester.pumpWidget(
        const MaterialApp(
          home: CometChatUsersWithMessages(),
        ),
      );

      await tester.pumpAndSettle(buffer);

      //verify if User data has been fetched
      final userItems = find.byType(CometChatListItem);

      expect(userItems, findsWidgets,
          reason: "CometChatUsers uses CometChatListItem to render view");

      print('user items found');

      final superHero2 =
          find.widgetWithText(CometChatListItem, "Captain America");

      // await tester.scrollUntilVisible(superHero2, 50, maxScrolls: 50);
      // expect(superHero2, findsOneWidget);
      // print('found you cap');

      final searchBar = find.byType(TextField);
      await tester.enterText(searchBar, "superhero2");
      await tester.pumpAndSettle(buffer);
      //check if text was accepted by textfield
      expect(find.widgetWithText(TextField, "superhero2"), findsOneWidget);
      //check for the list item containing superhero2
      expect(superHero2, findsOneWidget);

      print("found list item of Cap");

/*
      await tester.tap(userItems.first);
      // await tester.pump(Duration(seconds: 30));
      */

      await tester.tap(superHero2);
      await tester.pumpAndSettle(buffer);
      print('navigated to messages screen?');

      await tester.pumpAndSettle(buffer);

      //verify if really navigated to mesages screen
      final messageList = find.byType(CometChatMessageList);
      expect(messageList, findsOneWidget);
      print('message list found');

      final messageInputField = find.byType(TextFormField);
      expect(messageInputField, findsOneWidget);
      print('message input field found');
      String textMessage = "this is a test TextMessage 1";
      await tester.enterText(messageInputField, textMessage);
      await tester.pumpAndSettle(buffer);

      final updatedTextField = find.widgetWithText(TextFormField, textMessage);

      expect(updatedTextField, findsOneWidget);

      print('message input field found containing text $textMessage');
      final sendButton = find.widgetWithImage(
          IconButton,
          const AssetImage(AssetConstants.send,
              package: UIConstants.packageName));
      expect(sendButton, findsOneWidget);
      print('send button found');
      await tester.tap(sendButton);
      await tester.pumpAndSettle(buffer);
      print('message sent?');

      final textBubbles = find.byType(CometChatTextBubble);
      expect((textBubbles.evaluate().first.widget as CometChatTextBubble).text,
          textMessage);
      print(
          'text message displayed using the appropriate bubble in message list');
    });
  });
}
