import 'package:flutter/material.dart';
import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:url_launcher/url_launcher.dart';

///[LinkPreviewExtensionDecorator] is a the view model for [LinkPreviewExtension] it contains all the relevant business logic
///it is also a sub-class of [DataSourceDecorator] which allows any extension to override the default methods provided by [MessagesDataSource]
class LinkPreviewExtensionDecorator extends DataSourceDecorator {
  String messageTranslationTypeConstant = ExtensionConstants.linkPreview;
  LinkPreviewConfiguration? configuration;

  LinkPreviewExtensionDecorator(DataSource dataSource, {this.configuration})
      : super(dataSource);

  @override
  Widget getTextMessageContentView(TextMessage message, BuildContext context,
      BubbleAlignment _alignment, CometChatTheme theme) {
    Widget? child =
        super.getTextMessageContentView(message, context, _alignment, theme);
    return LinkPreviewBubble(
      theme: configuration?.theme ?? theme,
      onTapUrl: onTapUrl,
      links: getMessageLinks(message),
      child: child,
      defaultImage: configuration?.defaultImage,
      style: configuration?.style,
    );
  }

  @override
  String getId() {
    return "LinkPreview";
  }

  List<dynamic> getMessageLinks(BaseMessage message) {
    Map<String, Map>? extensionList =
        ExtensionModerator.extensionCheck(message);
    List<dynamic> links = [];
    if (extensionList != null) {
      try {
        if (extensionList.containsKey(ExtensionConstants.linkPreview)) {
          Map<dynamic, dynamic>? linkPreview =
              extensionList[ExtensionConstants.linkPreview];
          links = linkPreview?["links"] ?? [];
        }
      } catch (e) {
        debugPrint('$e');
      }
    }
    return links;
  }

  final RegExp _emailRegex = RegExp(
    r'^(.*?)((mailto:)?[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z][A-Z]+)',
    caseSensitive: false,
  );
  final RegExp _urlRegex =
      RegExp(r'(?:(?:https?|ftp):\/\/)?[\w/\-?=%.]+\.[\w/\-?=%.]+');

  final RegExp _phoneNumberRegex = RegExp(
      r'\b(\+?( |-|\.)?\d{1,2}( |-|\.)?)?(\(?\d{3}\)?|\d{3})( |-|\.)?(\d{3}( |-|\.)?\d{4})\b');

  Future<void> onTapUrl(String url) async {
    if (_urlRegex.hasMatch(url)) {
      if (!RegExp(r'^(https?:\/\/)').hasMatch(url)) {
        url = 'https://' + url;
      }
      await launch(url);
    } else if (_emailRegex.hasMatch(url)) {
      await launch('mailto:$url');
    } else if (_phoneNumberRegex.hasMatch(url)) {
      await launch('tel:$url');
    }
  }
}
